import bs
import bsTeamGame
import bsMap
import bsUtils
import bsSpaz
import bsScoreBoard
import bsFlag
import weakref
import random

def getGames():
    return [AssaultGame]

class AssaultGame(bsTeamGame.TeamGameActivity):

    @classmethod
    def getName(c):
        return 'Assault'

    @classmethod
    def getDescription(c,sessionType):
        return 'Reach the enemy flag to score.'

    @classmethod
    def getSupportedSessionTypes(self):
        return ['teams']

    @classmethod
    def getSupportedMaps(self,sessionType):
        return bsMap.getMapList("teamFlag")

    @classmethod
    def getSettings(self,sessionType):
        return [("Score to Win",{'minValue':1,'default':3}),
                ("Time Limit",{'choices':[('None',0),('1 Minute',60),
                                        ('2 Minutes',120),('5 Minutes',300),
                                        ('10 Minutes',600),('20 Minutes',1200)],
                               'default':0}),
                ("Respawn Times",{'choices':[('Shorter',0.25),('Short',0.5),('Normal',1.0),('Long',2.0),('Longer',4.0)],
                                  'default':1.0}),
                ("Epic Mode",{'default':False})]

    def __init__(self,session,level,settings):
        bsTeamGame.TeamGameActivity.__init__(self,session,level,settings)
        self._scoreBoard = bsScoreBoard.ScoreBoard(self)
        if self.settings['Epic Mode']: self._isSlowMotion = True
        self._lastScoreTime = 0
        self._scoreSound = bs.getSound("score")

    def getInstanceDescription(self):
        if self.settings['Score to Win'] == 1: return 'Touch the enemy flag.'
        else: return ('Touch the enemy flag ${ARG1} times.',self.settings['Score to Win'])
        
    def getInstanceScoreBoardDescription(self):
        if self.settings['Score to Win'] == 1: return 'touch 1 flag'
        else: return ('touch ${ARG1} flags',self.settings['Score to Win'])

    def onTransitionIn(self):
        bsTeamGame.TeamGameActivity.onTransitionIn(self, music='Epic' if self.settings['Epic Mode'] else 'ForwardMarch')

    def onTeamJoined(self,team):
        team.gameData['score'] = 0
        self._updateScoreBoard()

    def onBegin(self):
        bsTeamGame.TeamGameActivity.onBegin(self,nameSound=bs.getSound('announceAssault'))

        self.setupStandardTimeLimit(self.settings['Time Limit'])

        self._baseRegionMaterials = {}
        for team in self.teams.values():
            m = self._baseRegionMaterials[team.teamID] = bs.Material('Assault base material '+str(team.teamID))
            m.addAction(conditions=("dstIsMaterial",bs.playerMaterial),
                        actions=(("partMod","collide",True),
                                 ("partMod","physical",False),
                                 ("call","atConnect",bs.WeakCall(self._handleBaseCollide,weakref.ref(team)))))

        # create a score region and flag for each team
        for team in self.teams.values():
            team.gameData['basePos'] = self._level.getFlagPosition(team.teamID)

            bs.newNode("light",
                       owner=self,
                       attrs={"translate": team.gameData['basePos'],
                              "intensity":0.6,
                              "heightAttenuated":False,
                              "volumeIntensityScale":0.1,
                              "radius":0.1,
                              "color": team.color})

            self.projectFlagStand(team.gameData['basePos'])

            team.gameData['flag'] = bsFlag.Flag(self,bs.Actor.UNRETAINED,
                                                touchable=False,
                                                translate=team.gameData['basePos'],
                                                color=team.color)
            p = team.gameData['basePos']
            region = bs.newNode("region",
                                owner=team.gameData['flag'].node,
                                attrs={"translate":(p[0],p[1]+0.75,p[2]),
                                       "scale": (0.5,0.5,0.5),
                                       "type": "sphere",
                                       "materials":(self._baseRegionMaterials[team.teamID].materialID,)})

    def handleMessage(self,m):
        if isinstance(m,bsSpaz.PlayerSpazDeathMessage):
            bsTeamGame.TeamGameActivity.handleMessage(self,m) # augment standard
            self.respawnPlayer(m.spaz.getPlayer())
        else:
            bsTeamGame.TeamGameActivity.handleMessage(self,m)

    def _flashBase(self,team,length=2000):
        light = bs.newNode("light",
                           owner=self,
                           attrs={"translate": team.gameData['basePos'],
                                  "heightAttenuated": False,
                                  "radius":0.3,
                                  "color": team.color})
        bsUtils.animate(light,"intensity",{0:0,250:2.0,500:0},loop=True)
        bs.gameTimer(length,light.delete)

    def _handleBaseCollide(self,teamRef):
        team = teamRef()
        if team is None: return
        try: player = bs.getCollideValues("opposingNode").getDelegate().getPlayer()
        except Exception: return

        # if its another team's player, they scored
        playerTeam = player.getTeam()
        if playerTeam is not team:

            # (prevent multiple simultaneous scores)
            if bs.getGameTime() != self._lastScoreTime:
                self._lastScoreTime = bs.getGameTime()

                self.scoreSet.playerScored(player,50,bigMessage=True)

                bs.playSound(self._scoreSound)
                self._flashBase(team)

                # move all players on the scoring team back to their start
                # and add flashes of light so its noticable
                for p in playerTeam.players.values():
                    if p.isAlive():
                        pos = p.actor.node.translate
                        light = bs.newNode("light",
                                           owner=self,
                                           attrs={"translate":pos,
                                                  "color":playerTeam.color,
                                                  "heightAttenuated":False,
                                                  "radius":0.4})
                        bs.gameTimer(500,light.delete)
                        bsUtils.animate(light,'intensity',{0:0,100:1.0,500:0})

                        newPos = self._level.getStartPosition(playerTeam.teamID)
                        light = bs.newNode("light",
                                           owner=self,
                                           attrs={"translate":newPos,
                                                  "color":playerTeam.color,
                                                  "radius":0.4,
                                                  "heightAttenuated":False})
                        bs.gameTimer(500,light.delete)
                        bsUtils.animate(light,'intensity',{0:0,100:1.0,500:0})
                        p.actor.handleMessage(bs.StandMessage(newPos,random.uniform(0,360)))

                # have teammates celebrate
                for player in playerTeam.players.values():
                    try: player.actor.node.message('celebrate',2000)
                    except Exception: pass

                playerTeam.gameData['score'] += 1
                self._updateScoreBoard()
                if playerTeam.gameData['score'] >= self.settings['Score to Win']:
                    self.endGame()

    def endGame(self):
        results = bsTeamGame.Results(scoreName='Points')
        for t in self.teams.values(): results.setTeamScore(t,t.gameData['score'])
        self.end(results=results)
        
    def _updateScoreBoard(self):
        for team in self.teams.values():
            self._scoreBoard.setTeamValue(team,team.gameData['score'],self.settings['Score to Win'])

